#!/usr/bin/python
# -*- coding: utf-8 -*-

import os, sys, optparse

# command line parser
parser = optparse.OptionParser()

parser.add_option('-p','--productname',
    action='store',
    dest='productname',
    help="Name that will be given to the product")

parser.add_option('-t', '--target',
    action="store",
    dest="target",
    help="Target directory")

parser.add_option('--skindir',
    action='store',
    dest='skindir',
    help="Basename for the folders in the product 'skins/' directory")

parser.add_option('--skinname',
    action='store',
    dest='skinname',
    help="Name of the skin selection that will be added to 'portal_skins'")

parser.add_option('--stylesheet',
    action='store',
    dest='stylesheet',
    help="Name of the future stylesheet file")

parser.add_option('--enable-public-stylesheets',
    action='store_true',
    dest='enable_public',
    help="Keep default Plone public look.")

parser.add_option('--add-viewlet-example',
    action='store_true',
    dest='add_viewlet',
    help="Add a basic viewlet example to product.")

parser.add_option('--add-custom-setuphandler',
    action='store_true',
    dest='add_setuphandler',
    help="Add a basic import step to Generic Setup profile.")

parser.add_option('-v', '--verbose',
    action="store_true",
    dest="verbose",
    help="Verbose mode")

parser.add_option('-r', '--replace',
    action="store_true",
    dest="replace",
    help="Replace original")

options, args = parser.parse_args()

path = os.path.dirname( os.path.abspath(__file__) )
if path.endswith('%sbin' % os.sep):
    path = path[:-3]
else:
    raise IOError, 'This script must be in the DIYPloneStyle%sbin%s directory.' %(os.sep, os.sep)


# Output help if no argument is given while calling the script
if not options.productname and len(args) != 1 or not os.path.isdir(path):
    parser.parse_args(args=['-h'])

PRODUCTNAME = options.productname or args[0]
SKINDIR = options.skindir or PRODUCTNAME.lower()
skinname = options.skinname or PRODUCTNAME
SKINNAME = skinname.replace('"', "'")

# check for valid product name
if PRODUCTNAME[0] in '0123456789':
    raise IOError, "Product name cannot start with a digit"
else:
    pass

stylesheet = options.stylesheet or 'main.css'
if stylesheet.endswith('.dtml'):
    stylesheet = stylesheet[:-5]
if not stylesheet.endswith('.css'):
    stylesheet = '%s.css' % stylesheet
STYLESHEET = stylesheet

REPLACE_ORIGINAL = options.replace

PRODUCT_AUTHOR = 'John Doe <john.doe@dev.null>'

TEXT_FILES = {
    'HISTORY.txt': '''Changelog for %s

    (name of developer listed in brackets)

%s 1.0 - Unreleased

    - Generated skeleton product from DIYPloneStyle 3.0.
      [generator]
''' % ( PRODUCTNAME, PRODUCTNAME ),

    'README.txt': '''Description

    %(PRODUCTNAME)s is a product that adds a new style to a Plone 3.0.x portal.
    It adds a new skin selection to the 'portal_skins' tool
    (called %(SKINNAME)s), and registers a custom stylesheet (called 
    %(STYLESHEET)s) with the 'portal_css' tool.

    %(PRODUCTNAME)s is based on DIYPloneStyle 3.0, a skeleton product
    ready for building new graphical designs for Plone.

Installation

    On the file system: place %(PRODUCTNAME)s in the Products directory
    of your Zope instance and restart the server.

    In the Plone Web Interface: as portal manager, go to 'Portal > Site Setup
    > Add-on Products'.
    Select '%(PRODUCTNAME)s' and click the *Install* button.

    Uninstall -- Can be done from the same page.

Selecting a skin

    Depending on the values given in the skins tool profile (see
    profiles/default/skins.xml), the %(SKINNAME)s skin will be selected (or
    not) as default one while installing the product. If you need to switch
    from a default skin to another, go to the 'Site Setup' page, and choose
    'Themes' (as portal manager).

    Note -- Don't forget to perform a full refresh of the page or reload all
    images (not from browser cache) after selecting a skin. In Firefox, you
    can do so by pressing the 'shift' key while reloading the page. In IE, use
    the key combination <Ctrl-F5>.

Written by

    %(PRODUCT_AUTHOR)s
''' % locals(),

    'version.txt': '1.0',
    }

UNWANTED_TEXT_LINES = ()

UNWANTED_FOLDERS_AND_FILES = (
    'bin',
    'example',
    )

if options.enable_public:
    UNWANTED_FOLDERS_AND_FILES += (
        'base.css.dtml',
        'public.css.dtml',
        'generated.css.dtml',
        'portlets.css.dtml',
        )

if not options.add_viewlet:
    UNWANTED_FOLDERS_AND_FILES += (
        'templates',
        'viewlets.xml',
        'viewlets.py'
        )

if not options.add_setuphandler:
    UNWANTED_FOLDERS_AND_FILES += (
        'import_steps.xml',
        'setuphandlers.py',
        )

UNWANTED_FILE_EXTENSIONS = ('.pyc', '.pyo', '.swp', )
UNWANTED_FILE_PREFIXES = ('.', '#', )

FILE_MAPPING = (
    ('diystyle_styles', '%s_styles' % SKINDIR),
    ('diystyle_custom_images', '%s_custom_images' % SKINDIR),
    ('diystyle_custom_templates', '%s_custom_templates' % SKINDIR),
   )

CODE_EXTENSIONS = ('.py', '.dtml', '.props', '.xml', '.zcml', '.css')
CODE_MAPPING = FILE_MAPPING + (
    ('DIYPloneStyle Base', SKINNAME),
    ('DIY Style Base', SKINNAME),
    ('main.css', STYLESHEET),
    ('David Convent <davconvent@gmail.com>', PRODUCT_AUTHOR),
    ('diystyle', SKINDIR),
    ('DIYPloneStyle', PRODUCTNAME),
    ('diyplonestyle', PRODUCTNAME.lower()),
   )

def log(message):
    if options.verbose:
        print >> sys.stdout, message

# Remove directory
def rmdir(dirname):
    log('Remove directory %s' % dirname)
    if os.path.isdir( dirname ):
        for filename in os.listdir(dirname):
            filepath = os.path.join(dirname, filename)
            if os.path.isfile(filepath):
                os.remove(os.path.join(dirname, filename))
            else:
                rmdir(filepath)
        os.rmdir(dirname)

# Test if the item must be kept in the new product.
def isItemKept(item):
    if item in UNWANTED_FOLDERS_AND_FILES: return False
    for unwanted in UNWANTED_FILE_EXTENSIONS:
        if item.endswith(unwanted): return False
    for unwanted in UNWANTED_FILE_PREFIXES:
        if item.startswith(unwanted): return False
    return True

# Recursive copy of a folder content 
def generateDirectoryCopy(srcDirectory, dstDirectory):
    if not os.path.exists(dstDirectory):
        os.mkdir(dstDirectory)
        log('Created directory %s' % dstDirectory)
    for item in os.listdir(srcDirectory):
        if isItemKept(item):
            # Get the name of the copied item in the new product.
            dst_item = item
            for src, dst in FILE_MAPPING:
                if dst_item.endswith(src):
                    dst_item = dst_item.strip(src) + dst
            # Get source and dest paths
            src_path = os.path.join(srcDirectory, item)
            dst_path = os.path.join(dstDirectory, dst_item)
            # Read, modify and write to destination if item is a file
            if os.path.isfile(src_path):
                # Read source file
                f_src = open(src_path,'r')
                data = f_src.read()
                f_src.close()
                # Full text replacement
                if item in TEXT_FILES.keys():
                    data = TEXT_FILES[item]
                # Code subsitutions
                f,ext = os.path.splitext(item)
                if ext in CODE_EXTENSIONS or item in ('LICENSE.txt',):
                    # Remove unwanted lines
                    lines = [ line for line in data.splitlines()
                              if line.strip() not in UNWANTED_TEXT_LINES ]
                    if f_src.name.endswith('browser/configure.zcml') \
                                                and not options.add_viewlet:
                        del lines[30:39]
                    data = '\n'.join(lines)
                    # Replace PRODUCTNAME, SKINNAME, etc.
                    for orig, dest in CODE_MAPPING:
                        data = data.replace(orig, dest)
                # Write destination file
                f_dst = open(dst_path,'w')
                f_dst.write(data)
                f_dst.close()
                log('Created file %s' % dst_path)

            # Create and copy content if item is a folder
            elif os.path.isdir(src_path):
                if not os.path.exists(dst_path):
                    os.mkdir(dst_path)
                    log('Created directory %s' % dst_path)
                generateDirectoryCopy(src_path, dst_path)

targetpath = os.path.abspath( os.path.curdir )
if targetpath.strip('/') == path.strip('/'):
    targetpath = os.path.dirname( os.path.abspath( os.path.curdir ) )
target = options.target or targetpath+os.sep+PRODUCTNAME

generateDirectoryCopy( path, target )
log('Created %s based on DIYPloneStyle.' % PRODUCTNAME)

if REPLACE_ORIGINAL:
    rmdir( os.path.join(path.strip('DIYPloneStyle'), 'DIYPloneStyle') )
    log('Removed DIYPloneStyle.')

log('%s product generated successfully.' % PRODUCTNAME)
